<?php

namespace App\Models;

use App\Traits\IsTable;
use App\Traits\Search;
use Eloquent;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\DatabaseNotification;
use Illuminate\Notifications\DatabaseNotificationCollection;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Auth;
use Laravel\Sanctum\HasApiTokens;
use Livewire\WithPagination;
use Modules\SystemSettings\Entities\Council;
use Modules\UserManagement\Entities\MyBusiness;

/**
 * App\User
 *
 * @property int $id
 * @property string $username
 * @property string $password
 * @property string $surnames
 * @property string $firstname
 * @property int $employee_id
 * @property int $is_active
 * @property string|null $remember_token
 * @property Carbon|null $created_at
 * @property Carbon|null $updated_at
 * @property string $access_level
 * @property int $resetmode
 * @property-read Employee $employee
 * @property-read DatabaseNotificationCollection|DatabaseNotification[] $notifications
 * @property-read int|null $notifications_count
 * @property-read Collection|UserRole[] $userRoles
 * @property-read int|null $user_roles_count
 * @method static \Illuminate\Database\Eloquent\Builder|User newModelQuery()
 * @method static \Illuminate\Database\Eloquent\Builder|User newQuery()
 * @method static \Illuminate\Database\Eloquent\Builder|User query()
 * @method static \Illuminate\Database\Eloquent\Builder|User whereAccessLevel($value)
 * @method static \Illuminate\Database\Eloquent\Builder|User whereCreatedAt($value)
 * @method static \Illuminate\Database\Eloquent\Builder|User whereEmployeeId($value)
 * @method static \Illuminate\Database\Eloquent\Builder|User whereId($value)
 * @method static \Illuminate\Database\Eloquent\Builder|User whereIsActive($value)
 * @method static \Illuminate\Database\Eloquent\Builder|User wherePassword($value)
 * @method static \Illuminate\Database\Eloquent\Builder|User whereRememberToken($value)
 * @method static \Illuminate\Database\Eloquent\Builder|User whereResetmode($value)
 * @method static \Illuminate\Database\Eloquent\Builder|User whereUpdatedAt($value)
 * @method static \Illuminate\Database\Eloquent\Builder|User whereUsername($value)
 * @mixin Eloquent
 * @property-read Collection|\Laravel\Sanctum\PersonalAccessToken[] $tokens
 * @property-read int|null $tokens_count
 * @property string $email
 * @method static \Illuminate\Database\Eloquent\Builder|User whereEmail($value)
 */
class User extends Authenticatable
{
    use HasApiTokens, Notifiable, IsTable, WithPagination;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'username',
        'first_names',
        'employee_id',
        'password',
        'is_active',
        'council_id',
        'birth_date',
        'sex',
        'surnames',
        'marital_status',
        'telephone',
        'telephone_alt',
        'id_number',
        'nationality',
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $appends = ['last_payment'];
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];
    /**
     * @var mixed
     */

    public function hasPermission($permission){
        return (bool) $this->permissions->{$permission};
    }

    public function business():HasMany{
        return $this->hasMany(MyBusiness::class,'user_id');
    }

    public function getFullNameAttribute() : string
    {
        return  $this->first_names .' '. $this->surnames;
    }
    public function licences(){
        return $this->hasMany(LicenceApplication::class ,'user_id');
    }
    public function isUser(){
        return $this->hasPermission('user');
    }

    public function permissions(): HasOne
    {
        return $this->hasOne(UserRole::class,'user_id');
    }

    public function councilHistory():HasMany{
        return $this->hasMany(LastCouncilLoggedIn::class,'user_id');
    }

    public function council():BelongsTo{
        return $this->belongsTo(Council::class,'council_id');
    }

    public function billingAccounts():HasMany{
        return $this->hasMany(BillingAccounts::class,'user_id');
    }

    public function getUserPendingApprovals () {
        return AccountVerificationApplication::where('user_id', Auth::user()->id)
            ->where('status', '!=', 'approved')
            ->where('council_id', Auth::user()->council_id)
            ->where('account_no', 'like', '%' . $this->search . '%')
            ->orderBy($this->columns[$this->orderBy], $this->orderDirection)->paginate($this->paginationCount);
    }

    public function getUserVerifiedAccounts () {
        return BillingAccounts::where('user_id', Auth::user()->id)
            ->where('status', 'verified')
            ->where('council_id', Auth::user()->council_id)
            ->where('account_no', 'like', '%' . $this->search . '%')
            ->orderBy($this->columns[$this->orderBy], $this->orderDirection)->paginate($this->paginationCount);
    }

    public function getaccountsPayments () {
        return BillingAccounts::where('user_id', Auth::user()->id)
            ->where('status', 'verified')
            ->with('payments')
            ->where('council_id', Auth::user()->council_id)
            ->where('account_no', 'like', '%' . $this->search . '%')->get();
//            ->orderBy($this->columns[$this->orderBy], $this->orderDirection)->paginate($this->paginationCount);
    }

    public function getUserPayments () {
        return Payment::where('user_id', Auth::user()->id)
            ->where('status', 'verified')
            ->where('council_id', Auth::user()->council_id)
//            ->orWhere('account_no', 'like', '%' . $this->search . '%')
//            ->orWhere('amount_paid', 'like', '%' . $this->search . '%')
//            ->orWhere('paynow_reference', 'like', '%' . $this->search . '%')
            ->orderBy($this->columns[$this->orderBy], $this->orderDirection)->paginate($this->paginationCount);
    }

    public function getAllUserVerifiedAccounts () {
        return BillingAccounts::where('user_id', Auth::user()->id)
            ->where('status', 'verified')
            ->orWhere('account_no', 'like', '%' . $this->search . '%')
            ->orderBy($this->columns[$this->orderBy], $this->orderDirection)->paginate($this->paginationCount);
    }

    public function waitingListListings (): HasMany
    {
        return $this->hasMany(WaitingLisQS::class, 'user_id');
    }

    public function queries (): HasMany
    {
        return $this->hasMany(LoggedQuery::class, 'user_id');
    }

    public function accountPayments(): HasMany
    {
        return $this->hasMany(AccountPortalPayment::class, 'user_id');
    }

    public function getLastPaymentAttribute()
    {
        return $this->accountPayments()->orderBy("created_at","desc")->first();
    }
}
